// Ant.m
//--------------------------------------------------------------------
#import "Ant.h"
#import "ModelSwarm.h"
//--------------------------------------------------------------------

@implementation Ant

//--------------------------------------------------------------------

-  setEnvironment: mS
       ruleMaster: rp
    dataWarehouse: dW
        interface: it
 reproductionRate: (float) rR
             posX: (int) pX
             posY: (int) pY
        direction: (int) d
            color: (int) c
{

   TRACE3(printf("\n%8X Ant - entry setEnvironment: %8X\n",
		 (int)self,(int)mS);)
   TRACE3(printf("                           ruleMaster: %8X\n",(int)rp);)
   TRACE3(printf("                        dataWarehouse: %8X\n",(int)dW);)
   TRACE3(printf("                            interface: %8X\n",(int)it);)
   TRACE3(printf("                     reproductionRate: %f\n",rR);)
   TRACE3(printf("                                 posX: %d\n",pX);)
   TRACE3(printf("                                 posY: %d\n",pY);)
   TRACE3(printf("                            direction: %d\n",d);)
   TRACE3(printf("                                color: %d\n",c);)

   environment      = mS;
   ruleMaster       = rp;
   dataWarehouse    = dW;
   interface        = it;
   reproductionRate = rR;
               xPos = pX;
               yPos = pY;
          direction = d;
              color = c;

   TRACE3(printf("%8X Ant - exit setEnvironment %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- createEnd
{

   ModelSwarm * environ;
   id obj;

   TRACE3(printf("\n%8X Ant - entry createEnd\n",(int)self);)

   environ = (ModelSwarm *) environment;

   world      = [environ getWorld];
   traces     = [environ getTraces];
   worldXSize = (int) [world getSizeX];
   worldYSize = (int) [world getSizeY];

   obj = [super createEnd];

   TRACE3(printf("%8X Ant - exit createEnd %8X\n\n",
                 (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- move
{

   int t;

   TRACE3(printf("\n%8X Ant - entry move\n",(int)self);)

   t = [traces getValueAtX: xPos Y: yPos];
   [traces putValue: (t+1) % 2 atX: xPos Y: yPos];

   [world deleteObject: self atX: xPos Y: yPos];

   [interface encodeColor: color trace: t direction: direction];
   [ruleMaster applyRuleTo: dataWarehouse with: interface];
   direction = [interface decode];

   if (direction == 0) xPos++;
   if (direction == 1) yPos++;
   if (direction == 2) xPos--;
   if (direction == 3) yPos--;

   xPos = (xPos + worldXSize) % worldXSize;
   yPos = (yPos + worldYSize) % worldYSize;

   [world addFirst: self atX: xPos Y: yPos];

   TRACE3(printf("%8X Ant - exit move: %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- generate
{

   ModelSwarm * environ;
   Ant * anAnt;
   float rR=0;
   int x,y,d,c=2;

   TRACE3(printf("\n%8X Ant - entry generate\n",(int)self);)

   environ = (ModelSwarm *) environment;

   if (([uniformDblRand getDoubleWithMin: 0 withMax: 1] <
       reproductionRate)
       && ([environ getRoomForAnts] > 0))
   {

      x = xPos + 1;
      y = yPos + 1;
      d = [uniformIntRand getIntegerWithMin: 0 withMax: 3];

      if (([uniformDblRand getDoubleWithMin: 0 withMax: 1] <
         reproductionRate*reproductionRate*reproductionRate)
          && ([environ getRoomForQueens] > 0))
      {

         c  = 3;
         rR = reproductionRate;
         x = [uniformIntRand getIntegerWithMin: 0 withMax: worldXSize];
         y = [uniformIntRand getIntegerWithMin: 0 withMax: worldYSize];

      }

      x = (x + worldXSize) % worldXSize;
      y = (y + worldYSize) % worldYSize;

      anAnt = [Ant createBegin: [self getZone]];
      [anAnt setEnvironment: environment
                 ruleMaster: ruleMaster
              dataWarehouse: dataWarehouse
                  interface: interface
           reproductionRate: rR
                       posX: x
                       posY: y
                  direction: d
                      color: c];

      anAnt = [anAnt createEnd];

      [environ addAnt: anAnt];

   }

   TRACE3(printf("%8X Ant - exit generate %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- drawSelfOn: (id <Raster>)r
{

   TRACE3(printf("\n%8X Ant - entry drawSelfOn: %8X\n",
		 (int)self,(int)r);)

   [r drawPointX: xPos Y: yPos Color: color];

   TRACE3(printf("%8X Ant - exit drawSelfOn: %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- (int) getDirection
{

   TRACE3(printf("\n%8X Ant - entry getDirection\n",(int)self);)
   TRACE3(printf("%8X Ant - exit getDirection %d\n\n",
                 (int)self,direction);)

   return direction;
}

//--------------------------------------------------------------------

- (int) getPosX
{

   TRACE3(printf("\n%8X Ant - entry getPosX\n",(int)self);)
   TRACE3(printf("%8X Ant - exit getPosX %d\n\n",
                 (int)self,xPos);)

   return xPos;
}

//--------------------------------------------------------------------

- (int) getPosY
{

   TRACE3(printf("\n%8X Ant - entry getPosY\n",(int)self);)
   TRACE3(printf("%8X Ant - exit getPosY %d\n\n",
                 (int)self,yPos);)

   return yPos;
}

//--------------------------------------------------------------------

- (int) getColor
{

   TRACE3(printf("\n%8X Ant - entry getColor\n",(int)self);)
   TRACE3(printf("%8X Ant - exit getColor %d\n\n",
                 (int)self,color);)

   return color;
}

//--------------------------------------------------------------------

- print
{

   [dataWarehouse print];

   return self;
}

//--------------------------------------------------------------------

@end
