// ModelSwarm.m
//--------------------------------------------------------------------
#import "ModelSwarm.h"
//--------------------------------------------------------------------
@implementation ModelSwarm
//--------------------------------------------------------------------
// These methods provide access to the objects inside the ModelSwarm.
// These objects are the ones visible to other classes via message call.
// In theory we could just let other objects use Probes to read our
// state, but message access is frequently more convenient.
- getWorld
{

   TRACE1(printf("\n%8X - ModelSwarm entry getWorld\n",(int)self);)
   TRACE1(printf("%8X - ModelSwarm exit getWorld: %8X\n\n",
                 (int)self,(int)world);)

   return world;

}
//--------------------------------------------------------------------
- getTraces
{

   TRACE1(printf("\n%8X - ModelSwarm entry getTraces\n",(int)self);)
   TRACE1(printf("%8X - ModelSwarm exit getTraces: %8X\n\n",
                 (int)self,(int)traces);)

   return traces;

}
//--------------------------------------------------------------------
- getAntList
{

   TRACE1(printf("\n%8X - ModelSwarm entry getAntList\n",(int)self);)
   TRACE1(printf("%8X - ModelSwarm exit getAntList: %8X\n\n",
                 (int)self,(int)antList);)

   return antList;
}
//--------------------------------------------------------------------
- getAnAnt
{

   Ant * anAnt;

   TRACE1(printf("\n%8X - ModelSwarm entry getAnAnt\n",
                 (int)self);)

   anAnt = [antList atOffset: 0];
   attempts++;

   TRACE1(printf("%8X - ModelSwarm exit getAnAnt %8X\n\n",
                 (int)self,(int)anAnt);)

   return anAnt;
}
//--------------------------------------------------------------------
- (int) getAntPosX: aA
{

   int x;

   TRACE1(printf("\n%8X - ModelSwarm entry getAntPosX: %8X\n",
                 (int)self,(int)aA);)

   x = [aA getPosX];

   TRACE1(printf("%8X - ModelSwarm exit getAntPosX %d\n\n",
                 (int)self,x);)

   return x;

}

//--------------------------------------------------------------------

- (int) getAntPosY: aA
{

   int y;

   TRACE1(printf("\n%8X - ModelSwarm entry getAntPosY: %8X\n",
                 (int)self,(int)aA);)

   y = [aA getPosY];

   TRACE1(printf("%8X - ModelSwarm exit getAntPosY %d\n\n",
                 (int)self,y);)

   return y;

}

//--------------------------------------------------------------------

- (int) getAntDirection: aA
{

   int d;

   TRACE1(printf("\n%8X - ModelSwarm entry getAntDirection: %8X\n",
                 (int)self,(int)aA);)

   d = [aA getDirection];

   TRACE1(printf("%8X - ModelSwarm exit getAntDirection %d\n\n",
                 (int)self,d);)

   return d;

}

//--------------------------------------------------------------------

- (int) getAntColor: aA
{

   int c;

   TRACE1(printf("\n%8X - ModelSwarm entry getAntColor: %8X\n",
                 (int)self,(int)aA);)

   c = [aA getColor];

   TRACE1(printf("%8X - ModelSwarm exit getAntColor %d\n\n",
                 (int)self,c);)

   return c;

}

//--------------------------------------------------------------------

- (int) getNumberOfAnts
{

   int n;

   TRACE1(printf("\n%8X - ModelSwarm entry getNumberOfAnts\n",
                 (int)self);)

   n = [antList getCount];

   if (maxNumberOfQueens == 1) n--;

   if (n == 0) noAnts++;
   else noAnts = 0;

   TRACE1(printf("%8X - ModelSwarm exit getNumberOfAnts %d\n\n",
                 (int)self,n);)

   return n;

}

//--------------------------------------------------------------------

- (int) getNumberOfQueens
{

   TRACE1(printf("\n%8X - ModelSwarm entry getNumberOfQueens\n",
                 (int)self);)

   TRACE1(printf("%8X - ModelSwarm exit getNumberOfQueens %d\n\n",
                 (int)self,numberOfQueens);)

   return numberOfQueens;

}

//--------------------------------------------------------------------

- (int) getEatenAnts
{

   int n;
   TRACE1(printf("\n%8X - ModelSwarm entry getEatenAnts\n",(int)self);)

   n = eatenAnts;
   eatenAnts = 0;

   TRACE1(printf("%8X - ModelSwarm exit getEatenAnts %d\n\n",
                 (int)self,n);)

   return n;

}

//--------------------------------------------------------------------

- (int) getRoomForQueens
{

   TRACE1(printf("\n%8X - ModelSwarm entry getRoomForQueens\n",(int)self);)
   TRACE1(printf("%8X - ModelSwarm exit getRoomForQueens: %d\n\n",
                 (int)self,maxNumberOfQueens-numberOfQueens);)

   return maxNumberOfQueens-numberOfQueens;

}

//--------------------------------------------------------------------

- (int) getRoomForAnts
{

   int n;

   TRACE1(printf("\n%8X - ModelSwarm entry getRoomForAnts\n",(int)self);)

   n = maxNumberOfAnts - [antList getCount];

   TRACE1(printf("%8X - ModelSwarm exit getRoomForAnts: %d\n\n",
                 (int)self,n);)

   return n;

}

//--------------------------------------------------------------------

- addAnt: aA
{

   TRACE1(printf("\n%8X - ModelSwarm entry addAnt: %8X\n",
		 (int)self,(int)aA);)

   if (maxNumberOfQueens == 1) [antList addFirst: aA];
   else [antList addLast: aA];
   if ([aA getColor] == 3) numberOfQueens++;

   TRACE1(printf("%8X - ModelSwarm exit addAnt %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- dropAnt: aA
{

   TRACE1(printf("\n%8X - ModelSwarm entry dropAnt: %8X\n",
                 (int)self,(int)aA);)

   [antList remove: aA];
   if ([aA getColor] == 3) numberOfQueens--;
   [aA drop];
   eatenAnts++;
   successes++;

   TRACE1(printf("%8X - ModelSwarm exit dropAnt %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

+ createBegin: aZone
{

   ModelSwarm *obj;
   id <ProbeMap> probeMap;

   TRACE1(printf("\n%8X - ModelSwarm entry createBegin: %8X\n",
                  (int)self,(int)aZone);)

   // in createBegin, we set up the simulation parameters
   // First, call our superclass createBegin - the return value is the
   // allocated Swarm object.

   obj = [super createBegin: aZone];

   // Now fill in various simulation parameters with default values.

   obj->worldXSize           = 200;
   obj->worldYSize           = 200;
   obj->exAnteTraceProb      = 0.01;
   obj->reproductionRate     = 0.75;
   obj->maxNumberOfAnts      = 50;
   obj->initialGeneration    = 20;
   obj->maxNumberOfQueens    = 5;
   obj->verificationTreshold = 100;

   // Build a customized probe map. Without a probe map, the default
   // is to show all variables and messages. Here we choose to
   // customize the appearance of the probe, give a nicer interface.

   probeMap = [EmptyProbeMap createBegin: aZone];
   [probeMap setProbedClass: [self class]];
   probeMap = [probeMap createEnd];

   // Add in a bunch of variables, one per simulation parameter

   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "worldXSize" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "worldYSize" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "exAnteTraceProb" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "reproductionRate" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "initialGeneration" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "maxNumberOfAnts" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "maxNumberOfQueens" inClass: [self class]]];
   [probeMap addProbe: [probeLibrary getProbeForVariable:
    "verificationTreshold" inClass: [self class]]];

   // Now install our custom probeMap into the probeLibrary.

   [probeLibrary setProbeMap: probeMap For: [self class]];

   // We've created the Langton Ant simulator and initialized it.
   // return the id of the newly created Swarm

   TRACE1(printf("%8X - ModelSwarm exit createBegin: %8X\n\n",
                 (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- createEnd
{

   id obj;

   TRACE1(printf("\n%8X - ModelSwarm entry createEnd\n",(int)self);)

   numberOfQueens = 0;
   eatenAnts = attempts = successes = printFlag = noAnts = 0;
   obj = [super createEnd];

   TRACE1(printf("%8X - ModelSwarm exit createBegin: %8X\n\n",
                 (int)self,(int)obj);)

   return obj;

}

//--------------------------------------------------------------------

- buildObjects
{

   EH_Dump           * dump;
   CW_Shuffler       * shuffler;
   CW_RuleMaker      * aRuleMaker;
   CW_RuleMaster     * aRuleMaster;
   CW_DataWarehouse  * aDataWarehouse;
   CW_Interface      * anInterface;
   PH_ParmWarehouse  * antParmWarehouse;
   PH_ParmWarehouse  * antEaterParmWarehouse;
   PH_Reader         * reader;
   Ant               * anAnt;
   int x, y, d, i;


   TRACE1(printf("\n%8X - ModelSwarm entry buildObjects\n",(int)self);)

   dump     = [EH_Dump  createBegin: self];
   dump     = [dump     createEnd];

   reader   = [PH_Reader createBegin: self];
   reader   = [reader    createEnd];

   shuffler = [CW_Shuffler createBegin: self];
   shuffler = [shuffler    createEnd];

   antParmWarehouse = [PH_ParmWarehouse createBegin: self];
   antParmWarehouse = [antParmWarehouse   createEnd];
                      [antParmWarehouse   loadFromFile: "Ant.prm"];

   antEaterParmWarehouse = [PH_ParmWarehouse createBegin: self];
   antEaterParmWarehouse = [antEaterParmWarehouse   createEnd];
   [antEaterParmWarehouse  loadFromFile: "AntEater.prm"];

   aRuleMaker  = [CW_RuleMaker createBegin: self];
   aRuleMaker  = [aRuleMaker   createEnd];
                 [aRuleMaker   setDump:     dump];
                 [aRuleMaker   setShuffler: shuffler];

   aRuleMaster = [CW_RuleMaster createBegin:  self];
                 [aRuleMaster   setRuleMaker: aRuleMaker];
   aRuleMaster = [aRuleMaster   createEnd];
                 [aRuleMaster   setDump:      dump];
                 [aRuleMaster   setShuffler:  shuffler];


// Here we create a list to store ants, ants we'll be removed when
// they're eaten.

   antList   = [List create: self];

// Here we create the spaces to store information about traces and
// position of ants, to avoid errors we create two separate spaces for
// traces and ants
// traces we'll be initial set with random values

   traces = [TraceSpace createBegin: self];
   [traces setSizeX: worldXSize Y: worldYSize];
   traces = [traces createEnd];

   [traces seedTraceWithProb: exAnteTraceProb]; // ex ante traces

   world = [MoGrid2d create: self setSizeX: worldXSize Y: worldYSize];

// To handle rule's modification and new rule's discovery we'll use a
// rule maker. Note that only anteater, using a learning classifier,
// needs ruleMaker's services.

// Both anteater and ants need ruleMaster's services, note that the
// server we'll be the same for both kinds of object, the difference
// in behaviour are handled in the dataWarehouse's data.

// Classifier's parameters are stored in a separate object to optimize
// memory allocation, because they can be the same for several
// dataWarehouses and agents, also if here we've two dataWarehouses and
// two classifiersParms.

   anInterface = [CW_Interface createBegin:    self];
                 [anInterface  setEventsToMap: 4];
   anInterface = [anInterface  createEnd];

// Because the ant ruleMaster isn't learning the dataWarehouse can be
// the same for all ants.

   aDataWarehouse=[CW_DataWarehouse createBegin: self];
                  [aDataWarehouse setParmWarehouse: antParmWarehouse];
                  [aDataWarehouse setReader:        reader];
   aDataWarehouse=[aDataWarehouse   createEnd];

// Let's print ant's dataWarehouse for verification

   printf("\n Ant's rules are the following: \n");
   [aDataWarehouse print];
   printf("-----------------------------------------------------\n");

// here we create a set of agents, i. e. ants, with ants and queens
// accordingly with values specified in the initial probe map.

// First we create ants: the first we'll be queen, i. e. they'll be
// able to reproduce with specified rate, for the others reproduction
// rate we'll be set to 0.

   x = [uniformIntRand getIntegerWithMin: 0 withMax: worldXSize];
   y = [uniformIntRand getIntegerWithMin: 0 withMax: worldYSize];
   d = [uniformIntRand getIntegerWithMin: 0 withMax: 3];

   anAnt = [Ant createBegin: self];
   [anAnt    setEnvironment: self
                ruleMaster: aRuleMaster
             dataWarehouse: aDataWarehouse
                 interface: anInterface
          reproductionRate: reproductionRate
                      posX: x
                      posY: y
                 direction: d
                     color: 3];

   anAnt = [anAnt createEnd];
   [self addAnt: anAnt];

   for(i=0;i<initialGeneration;i++)
   {

      [antList forEach: M(move)];
      [antList forEach: M(generate)];

   }

// Here we create classifierParm and dataWarehouse for anteater

   aDataWarehouse=[CW_DataWarehouse createBegin: self];
   [aDataWarehouse setParmWarehouse: antEaterParmWarehouse];
   [aDataWarehouse setReader:        reader];
   aDataWarehouse=[aDataWarehouse   createEnd];

   anInterface = [CW_Interface createBegin:    self];
                 [anInterface  setEventsToMap: 4];
   anInterface = [anInterface  createEnd];

// Let's print initial contents of anteater's dataWarehouse

   printf("\n Anteater's rules before learning are the following: \n");
   [aDataWarehouse print];
   printf("-----------------------------------------------------\n");

// Here we create anteater

   anteater = [Anteater createBegin: self];
   [anteater setEnvironment: self
              dataWarehouse: aDataWarehouse
                  interface: anInterface
                 ruleMaster: aRuleMaster];

   anteater = [anteater createEnd];

   TRACE1(printf("%8X - ModelSwarm exit buildObjects: %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- buildActions
{
   // Create the list of simulation actions. We put these in an action
   // group, because we want these actions to be executed in a specific
   // order, but these steps should take no (simulated) time. The
   // M(foo) means "The message called <foo>". You can send a message
   // To a particular object, or ForEach object in a collection.

   TRACE1(printf("\n%8X - ModelSwarm entry buildActions\n",(int)self);)

// first the ants are allowed to reproduce itself, if they were queens
// then the anteater have to detect for an ant, she can be a queen
// all ants have to move
// finally anteater try to eat the ant, he can eat only if the
// the calculate position is the same of the ant's position.

   modelActions = [ActionGroup create: self];
   [modelActions createActionForEach: antList   message: M(generate)];
   [modelActions createActionTo: anteater       message: M(search)];
   [modelActions createActionForEach: antList   message: M(move)];
   [modelActions createActionTo: anteater       message: M(eat)];
   [modelActions createActionTo: self           message: M(verify)];


   // Then we create a schedule that executes the modelActions. modelActions
   // is an ActionGroup, by itself it has no notion of time. In order to
   // have it executed in time, we create a Schedule that says to use
   // the modelActions ActionGroup at particular times.
   // This schedule has a repeat interval of 1, it will loop every time step.
   // The action is executed at time 0 relative to the beginning of the loop.

   // This is a simple schedule, with only one action that is just
   // repeated every time. See mousetraps for more complicated schedules.

   modelSchedule = [Schedule createBegin: self];
   [modelSchedule setRepeatInterval: 1];
   modelSchedule = [modelSchedule createEnd];
   [modelSchedule at: 0 createAction: modelActions];

   TRACE1(printf("%8X - ModelSwarm exit buildActions: %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

- activateIn: swarmContext
{
   // Here, we activate the swarm in the context passed in
   // Then we activate our schedule in ourselves

   TRACE1(printf("\n%8X - ModelSwarm entry activateIn: %8X\n",
                 (int)self,(int)swarmContext);)

   [super activateIn: swarmContext];

   [modelSchedule activateIn: self];

   TRACE1(printf("%8X - ModelSwarm exit activateIn: %8X\n\n",
                 (int)self,(int)[self getSwarmActivity]);)

   return [self getSwarmActivity];

}

//--------------------------------------------------------------------

- verify
{

   TRACE1(printf("\n%8X - ModelSwarm entry verify\n",
                 (int)self);)

   if (attempts >= verificationTreshold)
   {

      printf("%d of the latest %d attempts were successfull\n",
             successes,attempts);

      if ((attempts == successes) && (printFlag == 0))
      {

     printf("\n Anteater's rules after learning are the following: \n");
     [anteater print];
     printf("-----------------------------------------------------\n");
         printFlag = 1;

      }

      attempts = successes = 0;
      //      [anteater print];  

   }

   if ((noAnts >= verificationTreshold) && (printFlag == 0))
   {

       printf("\nNo more ants - anteater's rules are: \n");
         [anteater print];
         printFlag = 1;

   }

   TRACE1(printf("%8X - ModelSwarm exit verify %8X\n\n",
                 (int)self,(int)self);)

   return self;

}

//--------------------------------------------------------------------

@end
