// PH_ParmWarehouse.m
//--------------------------------------------------------------------
#import "PH_ParmWarehouse.h"
@implementation PH_ParmWarehouse
//--------------------------------------------------------------------
- createEnd
{
   numberOfParms = 0;
   return [super createEnd];
}
//--------------------------------------------------------------------
- (int) loadFromFile: (char*) fP
{
   int rc=0;

   row = 0;
   numberOfParms = 0;

   rc = [self openFile: fP];
   if (rc != 0) return rc;

   rc = [self readInto: buffer withLength: 80];
   if (rc != 0)
   {
      [self closeFile];
      return -4;              //parmFile is empty
   }

   do
   {
      row++;
      rc = [self analyze];
      if (rc == 0)
      {
         currentParm = [self findParm: name];
         if (currentParm == -1) currentParm = [self add];
         if (currentParm == -1) rc = -8; //maximum parm number reached
         [self upd];
      }

      rc = [self readInto: buffer withLength: 80];
      if (rc == 2) rc = 0; //ignore truncated assignement

   } while (rc == 0);

   [self closeFile];
   if (rc != 1) return -5; //generic error in reading parmFile

   return 0;
}
//--------------------------------------------------------------------
- (int) setParm: (char*) nP fromFloat: (float*) fP;
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;   //Parameter not found

   floatValues[currentParm] = *fP;

   if (types[currentParm] != 'f') return 3; //wrong type
   else return 0;
}
//--------------------------------------------------------------------
- (int) setParm: (char*) nP fromInteger: (int*) iP;
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;   //Parameter not found

   intValues[currentParm] = *iP;

   if (types[currentParm] != 'i') return 3; //wrong type
   else return 0;
}
//--------------------------------------------------------------------
- (int) setParm: (char*) nP fromString: (char*) sP;
{
   int i,j=0;
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;

   i = strlen(sP);
   if (i > 63)
   {
      i = 63;
      j = 2;                              //assignement is truncated
   }

   strncpy(charValues+(currentParm*64),sP,i);
   i = i + currentParm*64;
   charValues[i] = '\0';

   if (types[currentParm] != 'c') return 3; //illegale type
   else return j;
}
//--------------------------------------------------------------------
- (int) addParm: (char*) nP fromFloat: (float*) fP;
{
   int i,j=0;

   if ([self findParm: nP] != -1) return -7;  //Duplicate parameter

   i = strlen(nP);
   if (i > 63) return -9;                     //Name is too long

   currentParm = [self add];
   if (currentParm == -1) return -8;          //maximum parm number

   strncpy(names+(currentParm*64),nP,i);
   i = i + currentParm*64;
   names[i] = '\0';

   floatValues[currentParm] = *fP;
   types[currentParm] = 'f';
   return j;
}
//--------------------------------------------------------------------
- (int) addParm: (char*) nP fromInteger: (int*) iP;
{
   int i,j=0;

   if ([self findParm: nP] != -1) return -7;  //Duplicate Parm

   i = strlen(nP);
   if (i > 63) return -9;                     //Name is too long

   currentParm = [self add];
   if (currentParm == -1) return -8;          //maximum parm number

   strncpy(names+(currentParm*64),nP,i);
   i = i + currentParm * 64;
   names[i] = '\0';

   intValues[currentParm] = *iP;

   types[currentParm] = 'i';
   return j;
}
//--------------------------------------------------------------------
- (int) addParm: (char*) nP fromString: (char*) sP;
{
   int i,j=0,rc=0;

   if ([self findParm: nP] != -1) return -7;  //Duplicate Parm

   i = strlen(nP);
   if (i > 63) return -9;                     //Name is too long

   j = strlen(sP);

   if (j > 63)
   {
      j  = 63;
      rc = 2;
   }

   currentParm = [self add];
   if (currentParm == -1) return -8;  //maximum parm number

   strncpy(names+(currentParm*64),nP,i);
   i = i + currentParm * 64;
   names[i] = '\0';

   strncpy(charValues+(currentParm*64),sP,j);
   j = j + currentParm * 64;
   names[i] = '\0';

   types[currentParm] = 'c';

   return 0;
}
//--------------------------------------------------------------------
- (int) getParm: (char*) nP intoFloat: (float*) fP;
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;         //Parm not found

   *fP = floatValues[currentParm];

   if (types[currentParm] != 'f') return 3; //illegale type
   else return 0;
}
//--------------------------------------------------------------------
- (int) getParm: (char*) nP intoInteger: (int*) iP;
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;

   *iP = intValues[currentParm];

   if (types[currentParm] != 'i') return 3; //illegale type
   else return 0;
}
//--------------------------------------------------------------------
- (int) getParm: (char*) nP intoString: (char*) sP
                            withLength: (int)   ll
{
   int i,rc=0;

   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;

   i = strlen(charValues+(currentParm*64));
   if (i > ll-1)
   {
      i = ll-1;
      rc = 2;
   }

   strncpy(sP,charValues+(currentParm*64),i);
   sP[i] = '\0';

   if (types[currentParm] != 'c') return 3;
   return rc;
}
//--------------------------------------------------------------------
- (int) getTypeOfParm: (char*) nP into: (char*) tP
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;

   *tP = types[currentParm];

   return 0;
}
//--------------------------------------------------------------------
- (int) setTypeOfParm: (char*) nP from: (char*) tP
{
   currentParm = [self findParm: nP];
   if (currentParm == -1) return -6;

   if ((*tP != 'c') && (*tP != 'i') && (*tP != 'f')) return -10;

   types[currentParm] = tP[0];

   return 0;
}
//--------------------------------------------------------------------
- print
{
   int i;
   printf("parameter list contains %d elements\n",numberOfParms);
   for(i=0;i<numberOfParms;i++)
   printf("%s %c %d %f %s\n",names+(i*64),types[i],intValues[i],
      floatValues[i],charValues+(i*64));
   return self;
}
//--------------------------------------------------------------------
//               PRIVATE METHODS
//--------------------------------------------------------------------
- (int) analyze;
{

   int i=0,positionInWork=0,nameLen=0,valueLen=0,comment=0,
       endOfBuffer=0,point=0,positionAfterPoint=0;
   int   iS=-1;
   float fS=-1.0;
   //The maximum allowed length for parameter name and input value is
   //63, to store both strings 128 bites are needed, because the
   //strings are closed by an '\0' charachter.
   char  work[128];
   work[0] = work[63] = '\0';

   //Let's analyze each byte in the buffer. Because comment is made
   //with two consecutive slashes we have to count them. The field
   //comment counts the number of consecutive slashes, each time the
   //value of a byte is not slash we have to reset the counter
   //The switch endOfBuffer is used to stop the loop

   do
   {
      switch(buffer[i])
      {
         case '\0': endOfBuffer = 1;                    //empty row
                    break;
         case '=' : nameLen = positionInWork;           //end of name
                    positionInWork = 64;
                    comment = 0;                        //reset slash
                    break;                              //control
         case ';' : valueLen = positionInWork - 64;     //end of value
                    endOfBuffer = 1;
                    break;
         case '/' : work[positionInWork++] = buffer[i]; //Single slash
                    comment++;                          //can be a valid
                    break;                              //value
         case ' ' : comment = 0;                        //spaces are
                    break;                              //ignored
         default:   work[positionInWork++] = buffer[i]; //each other
                    comment = 0;                        //value is kept
                    break;
      }

      //All values after a double slashe are ignored, i.e. the string
      //is closed after the last non slash charachter.

      if (comment == 2)
      {
         if(positionInWork > 64) valueLen = valueLen - 2;
         else                    nameLen  = nameLen  - 2;
         positionInWork = positionInWork - 2;
         endOfBuffer = 1;
      }

      i++;

   } while (endOfBuffer == 0);

   if (positionInWork == 0) return -1;

   if ((nameLen<1) || (nameLen>63) || (valueLen<1) || (valueLen>63))
   {
      printf("Parse error in row %d - row ignored\n",row);
      return -1;
   }

   strncpy(name,work,nameLen);
   name[nameLen] = '\0';                //To end the string
   strncpy(charValue,work+64,valueLen);
   charValue[valueLen] = '\0';          //To end the string

   i              = 0;
   work[0]        = '\0';               //To allow reusage if work
   endOfBuffer    = 0;                  //buffer
   positionInWork = 0;
   type           = '?';

   do
   {
      if(charValue[i] == '\0') endOfBuffer = 1;

      if(charValue[i] == '.') point = 1;

      if(((charValue[i] >='0')&&(charValue[i] <='9')) || //is a number
         ((charValue[i] =='-')&&(i==0)))                 //is negative
      {
         work[positionInWork++] = charValue[i];
         positionAfterPoint     = positionAfterPoint + point;
      }
      else if((charValue[i] != '.') && (charValue[i] != '\0'))
      {
         endOfBuffer = 1;
         type       = 'c';
         floatValue = -1;
         intValue   = -1;
      }

      i++;

   } while (endOfBuffer == 0);
   work[positionInWork] = '\0';

   if(positionInWork == 0) // the string contains only points
   {
      type       = 'c';
      floatValue = -1;
      intValue   = -1;
   }

   if((type == '?') && (positionAfterPoint > 0))      // is a float
   {
     if (work[0] == '-')
       {
	 floatValue = atof(work+1);
	 floatValue = floatValue * fS;
       }
     else floatValue = atof(work);
      intValue   = -1;
      for(i=0;i<positionAfterPoint;i++) floatValue=floatValue/10;
      type = 'f';
   }

   if(type == '?')                                 // is an integer
   {
     if (work[0] == '-')
       {
	 floatValue = atof(work+1);
	 floatValue = floatValue * fS;
	 intValue   = atol(work+1);
	 intValue   = intValue * iS;
       }
     else
       {
	 floatValue = atof(work);
	 intValue   = atol(work);
       }
      type       = 'i';
   }

   return 0;
}
//--------------------------------------------------------------------
- (int) findParm: (char*) nP
{
   int i=0;
   if(numberOfParms != 0) do
   {
      if(strcmp(names+(i*64),nP) == 0) return i;
      i++;
   } while (i<=numberOfParms);
   return -1;
}
//--------------------------------------------------------------------
- (int) add
{
   if (numberOfParms == 256) return -1;
   return numberOfParms++;
}
//--------------------------------------------------------------------
- (int) upd
{
   strcpy(names+(currentParm*64),name);
   types[currentParm] = type;
   intValues[currentParm] = intValue;
   floatValues[currentParm] = floatValue;
   strcpy(charValues+(currentParm*64),charValue);
   return 0;
}
//--------------------------------------------------------------------
// File handling
//--------------------------------------------------------------------
- (int) openFile: (char*) fP
{
   if((filePtr = fopen(fP,"r")) == NULL)
   {
      printf("%s file not found\n",fP);
      return -1;                              //file not found
   }
   return 0;
}
//--------------------------------------------------------------------
- (int) readInto: (char*) bP withLength: (int) ll
{
   int i,j=2;

   if(fgets(bP,ll,filePtr) == NULL) return 1; //EOF

   for(i=0;i<ll;i++) if(bP[i] == '\n')
   {
      bP[i] = '\0';
      j     = 0;
   }

   bP[ll-1] = '\0';

   return j;
}
//--------------------------------------------------------------------
- (int) closeFile
{
   fclose(filePtr);
   return 0;
}
//--------------------------------------------------------------------
@end
